library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity de2_wm8731_audio is
port (
    clk_chip : in std_logic;
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    test_mode : in std_logic;       --    Audio CODEC controller test mode
    audio_request : out std_logic_vector(15 downto 0);  --    Audio controller request new data
    data : in std_logic_vector(15 downto 0) := "0000000000000000";
	 address    : in  std_logic_vector(16 downto 0);
	 write      : in std_logic;
    chipselect : in std_logic;
	 irq : out std_logic;
  
    -- Audio interface signals
    AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
    AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out  std_logic;   --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
end  de2_wm8731_audio;

architecture rtl of de2_wm8731_audio is     

    signal lrck : std_logic;
    signal bclk : std_logic;
    
    signal lrck_divider : unsigned(15 downto 0); 
    signal bclk_divider : unsigned(11 downto 0);
    
    signal set_bclk : std_logic;
    signal set_lrck : std_logic;
    signal clr_bclk : std_logic;
    signal lrck_lat : std_logic;
    
    signal shift_out : unsigned(15 downto 0);
    signal sin_out     : unsigned(15 downto 0); 
	 signal sin_counter : unsigned(5 downto 0); 
	 
	 type ram_type is array(2047 downto 0) of std_logic_vector(15 downto 0);
	 signal RAM : ram_type;
	 signal ram_address : unsigned(16 downto 0);
	 signal a           : unsigned(15 downto 0);
--	 signal dataout : ram_type;

begin
   ram_address <= unsigned(address(16 downto 0));
  
    -- LRCK divider 
    -- Audio chip main clock is 18.432MHz / Sample rate 48KHz
    -- Divider is 18.432 MHz / 48KHz = 192 (X"C0")
    -- Left justify mode set by I2C controller
	 
  
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then 
        lrck_divider <= (others => '0');
      elsif lrck_divider = X"01A1"  then        -- "C0" minus 1
        lrck_divider <= X"0000";
      else 
        lrck_divider <= lrck_divider + 1;
      end if;
    end if;   
  end process;
  
  
  process (clk)
  begin
    if rising_edge(clk) then      
      if reset_n = '0' then 
        bclk_divider <= (others => '0');
      elsif bclk_divider = X"019" or set_lrck = '1'  then  
        bclk_divider <= X"000";
      else 
        bclk_divider <= bclk_divider + 1;
      end if;
    end if;
  end process;

  set_lrck <= '1' when lrck_divider = X"01A1" else '0';
    
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        lrck <= '0';
      elsif set_lrck = '1' then 
        lrck <= not lrck;
      end if;
    end if;
  end process;
    
  -- BCLK divider
  set_bclk <= '1' when bclk_divider(11 downto 0) = X"00D" else '0';
  clr_bclk <= '1' when bclk_divider(11 downto 0) = X"019" else '0';
  
  
  
  
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        bclk <= '0';
      elsif set_lrck = '1' or clr_bclk = '1' then
        bclk <= '0';
      elsif set_bclk = '1' then 
        bclk <= '1';
      end if;
    end if;
  end process;


	
  -- Audio data shift output
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        shift_out <= (others => '0');
      elsif set_lrck = '1' then
        if test_mode = '1' then 
          shift_out <= sin_out;
			 --audio_request <= "0000000000000001";
        else 
          shift_out <= unsigned(RAM(to_integer(a)));
			 
			 
			 --audio_request <= "0000000000000001";
        end if;
      elsif clr_bclk = '1' then 
        shift_out <= shift_out (14 downto 0) & '0';
      end if;
    end if;   
  end process;

    -- Audio outputs
    
    AUD_ADCLRCK  <= lrck;          
    AUD_DACLRCK  <= lrck;          
    AUD_DACDAT   <= shift_out(15); 
    AUD_BCLK     <= bclk;          

    -- Self test with Sin wave
    	
		process(clk_chip) 
	 begin 
	 if rising_edge(clk_chip) then
	   if a = x"07FF" then
		   irq <= '1';
		elsif write = '1' and chipselect = '1' then
		   RAM(to_integer(ram_address)) <= data;
		   irq <= '0';
		end if;
		end if;
		end process;
		
		
    process(clk)      
    begin
      if rising_edge(clk) then
        if reset_n = '0' then 
            sin_counter <= (others => '0');
        elsif lrck_lat = '1' and lrck = '0'  then 
			 if a = x"07FF" then
			     a <= x"0000"; 
--				  dataout <= RAM;
          else
               a <= a + 1;		
          end if;
        end if;
      end if;
    end process;
	 
	 
    process(clk)
    begin
      if rising_edge(clk) then
        lrck_lat <= lrck;
      end if;
    end process;

end rtl;



